"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var vue_1 = __importDefault(require("vue"));
var warning_1 = __importDefault(require("warning"));
var Suspense_1 = require("./Suspense");
var currentInstance_1 = require("./currentInstance");
var findSuspenseInstance_1 = __importDefault(require("./findSuspenseInstance"));
function Lazy(asyncFactory, props) {
    return vue_1.default.extend({
        name: 'VueSuspenseLazy',
        props: props,
        created: function () {
            var _this = this;
            asyncFactory.suspenseInstance = currentInstance_1.currentSuspenseInstance || (0, findSuspenseInstance_1.default)(this);
            if ((0, Suspense_1.has)(asyncFactory))
                return;
            (0, Suspense_1.add)(asyncFactory);
            if (asyncFactory.resolved) {
                asyncFactory.$$waiter.then(function () {
                    (0, Suspense_1.del)(asyncFactory);
                });
                return;
            }
            var promise = asyncFactory();
            asyncFactory.$$waiter = promise;
            promise
                .then(function (C) {
                // Compatible ES Module
                if (C.__esModule && C.default) {
                    C = C.default;
                }
                asyncFactory.resolved = C;
                // Trigger update
                _this.$forceUpdate();
            })
                .catch(function (err) {
                (0, warning_1.default)(process.env.NODE_ENV === 'production', err.message);
                (0, Suspense_1.del)(asyncFactory, err);
            });
        },
        updated: function () {
            (0, Suspense_1.del)(asyncFactory);
        },
        render: function (h) {
            var _this = this;
            // Fix context
            var slots = Object.keys(this.$slots)
                .reduce(function (arr, key) { return arr.concat(_this.$slots[key] || []); }, [])
                .map(function (vnode) {
                vnode.context = _this._self;
                return vnode;
            });
            return asyncFactory.resolved
                ? h(asyncFactory.resolved, {
                    on: this.$listeners,
                    // If there is no props definition, fall back to `this.$attrs`
                    props: props ? this.$props : this.$attrs,
                    // Passthrough scopedSlots
                    scopedSlots: this.$scopedSlots,
                    attrs: this.$attrs,
                }, slots)
                : this._e();
        },
    });
}
exports.default = Lazy;
//# sourceMappingURL=lazy.js.map